@@ -23,10 +23,13 @@ usage() {
     echo "            [-a i386 | **amd64** | armhf ]"
     echo "            [-b **haproxy** ]"
     echo "            [-c **~/.cache/image-create** | <cache directory> ]"
-    echo "            [-d **xenial** | trusty | <other release id> ]"
+    echo "            [-d **xenial**/**7** | trusty | <other release id> ]"
+    echo "            [-e]"
     echo "            [-h]"
     echo "            [-i **ubuntu** | fedora | centos | rhel ]"
+    echo "            [-n]"
     echo "            [-o **amphora-x64-haproxy** | <filename> ]"
+    echo "            [-p]"
     echo "            [-r <root password> ]"
     echo "            [-s **2** | <size in GB> ]"
     echo "            [-t **qcow2** | tar | vhd ]"
@@ -37,9 +40,12 @@ usage() {
     echo "        '-b' is the backend type (default: haproxy)"
     echo "        '-c' is the path to the cache directory (default: ~/.cache/image-create)"
     echo "        '-d' distribution release id (default on ubuntu: xenial)"
+    echo "        '-e' enable complete mandatory access control systems when available (default: permissive)"
     echo "        '-h' display this help message"
     echo "        '-i' is the base OS (default: ubuntu)"
+    echo "        '-n' disable sshd (default: enabled)"
     echo "        '-o' is the output image file name"
+    echo "        '-p' install amphora-agent from distribution packages (default: disabled)"
     echo "        '-r' enable the root account in the generated image (default: disabled)"
     echo "        '-s' is the image size to produce in gigabytes (default: 2)"
     echo "        '-t' is the image type (default: qcow2)"
@@ -74,7 +80,7 @@ if [ -z $OCTAVIA_REPO_PATH ]; then
 fi
 dib_enable_tracing=
 
-while getopts "a:b:c:d:hi:o:t:r:s:vw:x" opt; do
+while getopts "a:b:c:d:ehi:no:pt:r:s:vw:x" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
@@ -99,6 +105,9 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:x" opt; do
         d)
             AMP_DIB_RELEASE=$OPTARG
         ;;
+        e)
+            AMP_ENABLE_FULL_MAC_SECURITY=1
+        ;;
         h)
             usage
         ;;
@@ -112,17 +121,14 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:x" opt; do
                 exit 3
             fi
         ;;
+        n)
+            AMP_DISABLE_SSHD=1
+        ;;
         o)
             AMP_OUTPUTFILENAME=$(readlink -f $OPTARG)
         ;;
-        t)
-            AMP_IMAGETYPE=$OPTARG
-            if [ $AMP_IMAGETYPE != "qcow2" ] && \
-                [ $AMP_IMAGETYPE != "tar" ] && \
-                [ $AMP_IMAGETYPE != "vhd" ]; then
-                echo "Error: Unsupported image type " $AMP_IMAGETYPE " specified"
-                exit 3
-            fi
+        p)
+            export DIB_INSTALLTYPE_amphora_agent=package
         ;;
         r)
             AMP_ROOTPW=$OPTARG
@@ -134,6 +140,15 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:x" opt; do
                 exit 3
             fi
         ;;
+        t)
+            AMP_IMAGETYPE=$OPTARG
+            if [ $AMP_IMAGETYPE != "qcow2" ] && \
+                [ $AMP_IMAGETYPE != "tar" ] && \
+                [ $AMP_IMAGETYPE != "vhd" ]; then
+                echo "Error: Unsupported image type " $AMP_IMAGETYPE " specified"
+                exit 3
+            fi
+        ;;
         v)
             version
         ;;
@@ -164,8 +179,8 @@ AMP_BASEOS=${AMP_BASEOS:-"ubuntu"}
 
 if [ "$AMP_BASEOS" = "ubuntu" ]; then
     export DIB_RELEASE=${AMP_DIB_RELEASE:-"xenial"}
-else
-    export DIB_RELEASE=${AMP_DIB_RELEASE}
+elif [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"7"}
 fi
 
 AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$PWD/amphora-x64-haproxy"}
@@ -174,6 +189,15 @@ AMP_IMAGETYPE=${AMP_IMAGETYPE:-"qcow2"}
 
 AMP_IMAGESIZE=${AMP_IMAGESIZE:-2}
 
+AMP_DISABLE_SSHD=${AMP_DISABLE_SSHD:-0}
+
+AMP_ENABLE_FULL_MAC_SECURITY=${AMP_ENABLE_FULL_MAC_SECURITY:-0}
+
+if [ "$AMP_BASEOS" = "rhel" ] && [ "$AMP_IMAGESIZE" -lt 3 ]; then
+    echo "RHEL based amphora requires an image size of at least 3GB"
+    exit 1
+fi
+
 OCTAVIA_ELEMENTS_PATH=$OCTAVIA_REPO_PATH/elements
 
 if ! [ -d $OCTAVIA_ELEMENTS_PATH ]; then
@@ -197,12 +221,10 @@ else
 fi
 
 # For system-wide installs, DIB will automatically find the elements, so we only check local path
-DIB_ELEMENTS_PATH=${DIB_REPO_PATH:-${OCTAVIA_REPO_PATH%/*}/diskimage-builder/elements}
-
 if [ "$DIB_LOCAL_ELEMENTS_PATH" ]; then
-    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH:$DIB_LOCAL_ELEMENTS_PATH
+    export ELEMENTS_PATH=$OCTAVIA_ELEMENTS_PATH:$DIB_LOCAL_ELEMENTS_PATH
 else
-    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH
+    export ELEMENTS_PATH=$OCTAVIA_ELEMENTS_PATH
 fi
 
 export CLOUD_INIT_DATASOURCES=${CLOUD_INIT_DATASOURCES:-"ConfigDrive"}
@@ -240,7 +262,7 @@ fi
 
 # Make sure we have the required packages installed
 if [ "$platform" = 'NAME="Ubuntu"' ]; then
-    PKG_LIST="qemu kpartx git"
+    PKG_LIST="qemu git"
     for pkg in $PKG_LIST; do
         if ! dpkg --get-selections | grep -q "^$pkg[[:space:]]*install$" >/dev/null; then
             echo "Required package " $pkg " is not installed.  Exiting."
@@ -256,64 +278,28 @@ if [ "$platform" = 'NAME="Ubuntu"' ]; then
             echo "Earlier versions don't support the extended attributes required."
             exit 1
     fi
-
-elif [ "$platform" = 'NAME=Fedora' ]; then
-    PKG_LIST="qemu kpartx git"
+elif [[ $platform =~ "SUSE" ]]; then
+    # OpenSUSE
+    # use rpm -q to check for qemu-tools and git-core
+    PKG_LIST="qemu-tools git-core"
     for pkg in $PKG_LIST; do
-        if ! yum list installed $pkg &> /dev/null; then
-            echo "Required package " $pkg " is not installed.  Exiting."
+        if ! rpm -q $pkg &> /dev/null; then
+            echo "Required package " ${pkg/\*} " is not installed.  Exiting."
             exit 1
         fi
     done
 else
-    # centos or rhel
-        PKG_LIST="qemu-kvm qemu-img"
-        for pkg in $PKG_LIST; do
-            if ! yum list installed $pkg &> /dev/null; then
-                if ! yum list installed $pkg"-ev" &> /dev/null; then
-                    echo "Required package " $pkg " or " $pkg"-ev" " is not installed.  Exiting."
-                    exit 1
-                fi
-            fi
-        done
-        PKG_LIST="kpartx git"
-        for pkg in $PKG_LIST; do
-            if ! yum list installed $pkg &> /dev/null; then
-                echo "Required package " $pkg " is not installed.  Exiting."
-                exit 1
-            fi
-        done
-
-        if [ ${platform:0:6} = "CentOS" ]; then
-            # install EPEL repo, in order to install argparse
-            PKG_LIST="python-argparse"
-            if ! yum list installed $pkg &> /dev/null; then
-                echo "CentOS requires the python-argparse package be "
-                echo "installed separately from the EPEL repo."
-                echo "Required package " $pkg " is not installed.  Exiting."
-                exit 1
-            fi
-        fi
-fi
-
-# pip may not be installed from package managers
-# only check that we find an executable
-if ! which pip &> /dev/null; then
-    echo "Required executable pip not found.  Exiting."
-    exit 1
-fi
-
-# "pip freeze" does not show argparse, even if it is explicitly installed,
-# because it is part of the standard python library in 2.7.
-# See https://github.com/pypa/pip/issues/1570
-
-PKG_LIST="Babel dib-utils PyYAML"
+    # fedora/centos/rhel
+    # Actual qemu-img name may be qemu-img, qemu-img-ev, qemu-img-rhev, ...
+    # "yum install qemu-img" works for all, but search requires wildcard
+    PKG_LIST="qemu-img* git"
     for pkg in $PKG_LIST; do
-        if ! pip freeze 2>/dev/null| grep -q "^$pkg==" &>/dev/null; then
-            echo "Required python package " $pkg " is not installed.  Exiting."
+        if ! yum info installed $pkg &> /dev/null; then
+            echo "Required package " ${pkg/\*} " is not installed.  Exiting."
             exit 1
         fi
     done
+fi
 
 if  [ "$AMP_WORKING_DIR" ]; then
     mkdir -p $AMP_WORKING_DIR
@@ -325,54 +311,46 @@ pushd $TEMP > /dev/null
 
 # Setup the elements list
 
-if [ "$AMP_BASEOS" = "ubuntu" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm ubuntu"}
-    AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND-ubuntu"
-    if [ "$BASE_OS_MIRROR" ]; then
-        AMP_element_sequence="$AMP_element_sequence apt-mirror"
-        export UBUNTU_MIRROR="$BASE_OS_MIRROR"
-    fi
-elif [ "$AMP_BASEOS" = "fedora" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm fedora selinux-permissive"}
-    AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
-    if [ "$BASE_OS_MIRROR" ]; then
-        AMP_element_sequence="$AMP_element_sequence fedora-mirror"
-        export FEDORA_MIRROR="$BASE_OS_MIRROR"
-    fi
-elif [ "$AMP_BASEOS" = "centos" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm centos7 epel selinux-permissive"}
-    AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
-    if [ "$BASE_OS_MIRROR" ]; then
-        AMP_element_sequence="$AMP_element_sequence centos-mirror"
-        export CENTOS_MIRROR="$BASE_OS_MIRROR"
-    fi
-elif [ "$AMP_BASEOS" = "rhel" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm rhel7 selinux-permissive"}
-    AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
+AMP_element_sequence=${AMP_element_sequence:-"base vm"}
+if [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    AMP_element_sequence="$AMP_element_sequence ${AMP_BASEOS}${DIB_RELEASE}"
+else
+    AMP_element_sequence="$AMP_element_sequence ${AMP_BASEOS}"
 fi
 
+# Add our backend element (haproxy, etc.)
+AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
+
 if [ "$AMP_ROOTPW" ]; then
     AMP_element_sequence="$AMP_element_sequence root-passwd"
     export DIB_PASSWORD=$AMP_ROOTPW
 fi
 
-# Add the Octavia keepalived, Amphora Agent and Pyroute elements
-if [ "$AMP_BASEOS" = "ubuntu" ]; then
-    AMP_element_sequence="$AMP_element_sequence rebind-sshd"
-    AMP_element_sequence="$AMP_element_sequence no-resolvconf"
-    AMP_element_sequence="$AMP_element_sequence amphora-agent-ubuntu"
-    AMP_element_sequence="$AMP_element_sequence keepalived-octavia-ubuntu"
-else
-    AMP_element_sequence="$AMP_element_sequence no-resolvconf"
-    AMP_element_sequence="$AMP_element_sequence amphora-agent"
-    AMP_element_sequence="$AMP_element_sequence keepalived-octavia"
+# Add the Amphora Agent and Pyroute elements
+AMP_element_sequence="$AMP_element_sequence rebind-sshd"
+AMP_element_sequence="$AMP_element_sequence no-resolvconf"
+AMP_element_sequence="$AMP_element_sequence amphora-agent"
+
+if [ "$AMP_ENABLE_FULL_MAC_SECURITY" -ne 1 ]; then
+    # SELinux systems
+    if [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+        AMP_element_sequence="$AMP_element_sequence selinux-permissive"
+    fi
 fi
 
+# Add keepalived-octavia element
+AMP_element_sequence="$AMP_element_sequence keepalived-octavia"
+
 # Add pip-cache element
 AMP_element_sequence="$AMP_element_sequence pip-cache"
 
-# Add certificate ramfs ecrypt element
-AMP_element_sequence="$AMP_element_sequence cert-ramfs-ecrypt"
+# Add certificate ramfs element
+AMP_element_sequence="$AMP_element_sequence certs-ramfs"
+
+# Disable SSHD if requested
+if [ "$AMP_DISABLE_SSHD" -eq 1 ]; then
+    AMP_element_sequence="$AMP_element_sequence remove-sshd"
+fi
 
 # Allow full elements override
 if [ "$DIB_ELEMENTS" ]; then
