@@ -28,7 +28,7 @@ import rfc3986
 from octavia.common import constants
 from octavia.common import exceptions
 from octavia.common import utils
-
+from octavia.i18n import _
 
 CONF = cfg.CONF
 
@@ -55,8 +55,7 @@ def url_path(url_path):
         if (
             p_url.scheme or p_url.userinfo or p_url.host or
             p_url.port or
-            p_url.path is None or
-            not p_url.path.startswith('/')
+            p_url.path is None
         ):
             raise exceptions.InvalidURLPath(url_path=url_path)
     except Exception:
@@ -235,6 +234,15 @@ def port_exists(port_id):
     return port
 
 
+def check_port_in_use(port):
+    """Raise an exception when a port is used."""
+    if port.device_id:
+        raise exceptions.ValidationException(detail=_(
+            "Port %(port_id)s is already used by device %(device_id)s ") %
+            {'port_id': port.id, 'device_id': port.device_id})
+    return False
+
+
 def subnet_exists(subnet_id):
     """Raises an exception when a subnet does not exist."""
     network_driver = utils.get_network_driver()
@@ -245,6 +253,16 @@ def subnet_exists(subnet_id):
     return subnet
 
 
+def qos_policy_exists(qos_policy_id):
+    network_driver = utils.get_network_driver()
+    try:
+        qos_policy = network_driver.get_qos_policy(qos_policy_id)
+    except Exception:
+        raise exceptions.InvalidSubresource(resource='qos_policy',
+                                            id=qos_policy_id)
+    return qos_policy
+
+
 def network_exists_optionally_contains_subnet(network_id, subnet_id=None):
     """Raises an exception when a network does not exist.
 
