@@ -69,12 +69,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             constants.ROLE_MASTER + '-' +
             constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
             constants.GENERATE_SERVER_PEM,
-            constants.GENERATE_SERVER_PEM_TASK,
-            constants.FAILOVER_AMPHORA_FLOW + '-' +
-            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
-            constants.GENERATE_SERVER_PEM,
-            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
-            constants.UPDATE_CERT_EXPIRATION)
+            constants.GENERATE_SERVER_PEM_TASK)
 
         super(ControllerWorker, self).__init__()
 
@@ -382,6 +377,34 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             delete_member_tf.run()
 
+    def batch_update_members(self, old_member_ids, new_member_ids,
+                             updated_members):
+        old_members = [self._member_repo.get(db_apis.get_session(), id=mid)
+                       for mid in old_member_ids]
+        new_members = [self._member_repo.get(db_apis.get_session(), id=mid)
+                       for mid in new_member_ids]
+        updated_members = [
+            (self._member_repo.get(db_apis.get_session(), id=m.get('id')), m)
+            for m in updated_members]
+        if old_members:
+            pool = old_members[0].pool
+        elif new_members:
+            pool = new_members[0].pool
+        else:
+            pool = updated_members[0][0].pool
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
+
+        batch_update_members_tf = self._taskflow_load(
+            self._member_flows.get_batch_update_members_flow(
+                old_members, new_members, updated_members),
+            store={constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer,
+                   constants.POOL: pool})
+        with tf_logging.DynamicLoggingListener(batch_update_members_tf,
+                                               log=LOG):
+            batch_update_members_tf.run()
+
     def update_member(self, member_id, member_updates):
         """Updates a pool member.
 
@@ -560,13 +583,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         l7rule = self._l7rule_repo.get(db_apis.get_session(),
                                        id=l7rule_id)
-
-        listeners = [l7rule.l7policy.listener]
-        load_balancer = l7rule.l7policy.listener.load_balancer
+        l7policy = l7rule.l7policy
+        listeners = [l7policy.listener]
+        load_balancer = l7policy.listener.load_balancer
 
         create_l7rule_tf = self._taskflow_load(
             self._l7rule_flows.get_create_l7rule_flow(),
             store={constants.L7RULE: l7rule,
+                   constants.L7POLICY: l7policy,
                    constants.LISTENERS: listeners,
                    constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(create_l7rule_tf,
@@ -582,13 +606,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         l7rule = self._l7rule_repo.get(db_apis.get_session(),
                                        id=l7rule_id)
-
-        load_balancer = l7rule.l7policy.listener.load_balancer
-        listeners = [l7rule.l7policy.listener]
+        l7policy = l7rule.l7policy
+        load_balancer = l7policy.listener.load_balancer
+        listeners = [l7policy.listener]
 
         delete_l7rule_tf = self._taskflow_load(
             self._l7rule_flows.get_delete_l7rule_flow(),
             store={constants.L7RULE: l7rule,
+                   constants.L7POLICY: l7policy,
                    constants.LISTENERS: listeners,
                    constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(delete_l7rule_tf,
@@ -605,13 +630,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         l7rule = self._l7rule_repo.get(db_apis.get_session(),
                                        id=l7rule_id)
-
-        listeners = [l7rule.l7policy.listener]
-        load_balancer = l7rule.l7policy.listener.load_balancer
+        l7policy = l7rule.l7policy
+        listeners = [l7policy.listener]
+        load_balancer = l7policy.listener.load_balancer
 
         update_l7rule_tf = self._taskflow_load(
             self._l7rule_flows.get_update_l7rule_flow(),
             store={constants.L7RULE: l7rule,
+                   constants.L7POLICY: l7policy,
                    constants.LISTENERS: listeners,
                    constants.LOADBALANCER: load_balancer,
                    constants.UPDATE_DICT: l7rule_updates})
@@ -619,6 +645,53 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_l7rule_tf.run()
 
+    def _perform_amphora_failover(self, amp, priority):
+        """Internal method to perform failover operations for an amphora.
+
+        :param amp: The amphora to failover
+        :param priority: The create priority
+        :returns: None
+        """
+
+        stored_params = {constants.FAILED_AMPHORA: amp,
+                         constants.LOADBALANCER_ID: amp.load_balancer_id,
+                         constants.BUILD_TYPE_PRIORITY: priority, }
+
+        if amp.status == constants.DELETED:
+            LOG.warning('Amphora %s is marked DELETED in the database but '
+                        'was submitted for failover. Marking it busy in the '
+                        'amphora health table to exclude it from health '
+                        'checks and skipping the failover.', amp.id)
+            self._amphora_health_repo.update(db_apis.get_session(), amp.id,
+                                             busy=True)
+            return
+
+        if (CONF.house_keeping.spare_amphora_pool_size == 0) and (
+                CONF.nova.enable_anti_affinity is False):
+            LOG.warning("Failing over amphora with no spares pool may "
+                        "cause delays in failover times while a new "
+                        "amphora instance boots.")
+
+        # if we run with anti-affinity we need to set the server group
+        # as well
+        if CONF.nova.enable_anti_affinity:
+            lb = self._amphora_repo.get_all_lbs_on_amphora(
+                db_apis.get_session(), amp.id)
+            if lb:
+                stored_params[constants.SERVER_GROUP_ID] = (
+                    lb[0].server_group_id)
+
+        failover_amphora_tf = self._taskflow_load(
+            self._amphora_flows.get_failover_flow(role=amp.role,
+                                                  status=amp.status),
+            store=stored_params)
+
+        with tf_logging.DynamicLoggingListener(
+                failover_amphora_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
+
+            failover_amphora_tf.run()
+
     def failover_amphora(self, amphora_id):
         """Perform failover operations for an amphora.
 
@@ -626,49 +699,63 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises AmphoraNotFound: The referenced amphora was not found
         """
-
         try:
             amp = self._amphora_repo.get(db_apis.get_session(),
                                          id=amphora_id)
-            stored_params = {constants.FAILED_AMPHORA: amp,
-                             constants.LOADBALANCER_ID: amp.load_balancer_id,
-                             constants.BUILD_TYPE_PRIORITY:
-                                 constants.LB_CREATE_FAILOVER_PRIORITY}
-
-            if amp.status == constants.DELETED:
-                LOG.warning('Amphora %s is marked DELETED in the database '
-                            'but was submitted for failover. Marking it busy '
-                            'in the amphora health table to exclude it from '
-                            'health checks and skipping the failover.',
-                            amp.id)
-                self._amphora_health_repo.update(db_apis.get_session(), amp.id,
-                                                 busy=True)
-                return
-
-            # if we run with anti-affinity we need to set the server group
-            # as well
-            if CONF.nova.enable_anti_affinity:
-                lb = self._amphora_repo.get_all_lbs_on_amphora(
-                    db_apis.get_session(), amp.id)
-                if lb:
-                    stored_params[constants.SERVER_GROUP_ID] = (
-                        lb[0].server_group_id)
-
-            failover_amphora_tf = self._taskflow_load(
-                self._amphora_flows.get_failover_flow(
-                    role=amp.role,
-                    status=amp.status),
-                store=stored_params)
-            with tf_logging.DynamicLoggingListener(
-                    failover_amphora_tf, log=LOG,
-                    hide_inputs_outputs_of=self._exclude_result_logging_tasks):
-
-                failover_amphora_tf.run()
-
+            self._perform_amphora_failover(
+                amp, constants.LB_CREATE_FAILOVER_PRIORITY)
+            if amp.load_balancer_id:
+                LOG.info("Mark ACTIVE in DB for load balancer id: %s",
+                         amp.load_balancer_id)
+                self._lb_repo.update(
+                    db_apis.get_session(), amp.load_balancer_id,
+                    provisioning_status=constants.ACTIVE)
         except Exception as e:
             with excutils.save_and_reraise_exception():
                 LOG.error("Failover exception: %s", e)
 
+    def failover_loadbalancer(self, load_balancer_id):
+        """Perform failover operations for a load balancer.
+
+        :param load_balancer_id: ID for load balancer to failover
+        :returns: None
+        :raises LBNotFound: The referenced load balancer was not found
+        """
+
+        # Note: This expects that the load balancer is already in
+        #       provisioning_status=PENDING_UPDATE state
+        try:
+            lb = self._lb_repo.get(db_apis.get_session(),
+                                   id=load_balancer_id)
+
+            amps = lb.amphorae
+            for amp in amps:
+                # failover amphora in backup role
+                # Note: this amp may not currently be the backup
+                # TODO(johnsom) Change this to query the amp state
+                #               once the amp API supports it.
+                if amp.role == constants.ROLE_BACKUP:
+                    self._perform_amphora_failover(
+                        amp, constants.LB_CREATE_ADMIN_FAILOVER_PRIORITY)
+
+            for amp in amps:
+                # failover everyhting else
+                if amp.role != constants.ROLE_BACKUP:
+                    self._perform_amphora_failover(
+                        amp, constants.LB_CREATE_ADMIN_FAILOVER_PRIORITY)
+
+            self._lb_repo.update(
+                db_apis.get_session(), load_balancer_id,
+                provisioning_status=constants.ACTIVE)
+
+        except Exception as e:
+            with excutils.save_and_reraise_exception():
+                LOG.error("LB %(lbid)s failover exception: %(exc)s",
+                          {'lbid': load_balancer_id, 'exc': e})
+                self._lb_repo.update(
+                    db_apis.get_session(), load_balancer_id,
+                    provisioning_status=constants.ERROR)
+
     def amphora_cert_rotation(self, amphora_id):
         """Perform cert rotation for an amphora.
 
