@@ -48,10 +48,9 @@ class BaseComputeTask(task.Task):
 class ComputeCreate(BaseComputeTask):
     """Create the compute instance for a new amphora."""
 
-    def execute(self, amphora_id,
+    def execute(self, amphora_id, config_drive_files=None,
                 build_type_priority=constants.LB_CREATE_NORMAL_PRIORITY,
-                ports=None, config_drive_files=None,
-                server_group_id=None):
+                server_group_id=None, ports=None):
         """Create an amphora
 
         :returns: an amphora
@@ -63,9 +62,12 @@ class ComputeCreate(BaseComputeTask):
         LOG.debug("Compute create execute for amphora with id %s", amphora_id)
 
         user_data_config_drive = CONF.controller_worker.user_data_config_drive
+
+        key_name = CONF.controller_worker.amp_ssh_key_name
+        # TODO(rm_work): amp_ssh_access_allowed is deprecated in Pike.
+        # Remove the following two lines in the S release.
         ssh_access = CONF.controller_worker.amp_ssh_access_allowed
-        ssh_key = CONF.controller_worker.amp_ssh_key_name
-        key_name = None if not ssh_access else ssh_key
+        key_name = None if not ssh_access else key_name
 
         try:
             if CONF.haproxy_amphora.build_rate_limit != -1:
@@ -124,7 +126,7 @@ class ComputeCreate(BaseComputeTask):
 class CertComputeCreate(ComputeCreate):
     def execute(self, amphora_id, server_pem,
                 build_type_priority=constants.LB_CREATE_NORMAL_PRIORITY,
-                ports=None, server_group_id=None):
+                server_group_id=None, ports=None):
         """Create an amphora
 
         :returns: an amphora
@@ -137,9 +139,10 @@ class CertComputeCreate(ComputeCreate):
             '/etc/octavia/certs/server.pem': server_pem,
             '/etc/octavia/certs/client_ca.pem': ca}
         return super(CertComputeCreate, self).execute(
-            amphora_id, build_type_priority=build_type_priority,
-            ports=ports, config_drive_files=config_drive_files,
-            server_group_id=server_group_id)
+            amphora_id, config_drive_files=config_drive_files,
+            build_type_priority=build_type_priority,
+            server_group_id=server_group_id, ports=ports,
+            )
 
 
 class DeleteAmphoraeOnLoadBalancer(BaseComputeTask):
