@@ -21,6 +21,7 @@ reference
 import datetime
 
 from oslo_config import cfg
+from oslo_db import api as oslo_db_api
 from oslo_db import exception as db_exception
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -78,7 +79,8 @@ class BaseRepository(object):
     def delete_batch(self, session, ids=None):
         """Batch deletes by entity ids."""
         ids = ids or []
-        [self.delete(session, id) for id in ids]
+        for id in ids:
+            self.delete(session, id=id)
 
     def update(self, session, id, **model_kwargs):
         """Updates an entity in the database.
@@ -118,8 +120,12 @@ class BaseRepository(object):
         query = query.options(joinedload('*'))
 
         if not deleted:
-            query = query.filter(
-                self.model_class.provisioning_status != consts.DELETED)
+            if hasattr(self.model_class, 'status'):
+                query = query.filter(
+                    self.model_class.status != consts.DELETED)
+            else:
+                query = query.filter(
+                    self.model_class.provisioning_status != consts.DELETED)
 
         if pagination_helper:
             model_list, links = pagination_helper.apply(
@@ -170,7 +176,7 @@ class Repositories(object):
         :param session: A Sql Alchemy database session.
         :param lb_dict: Dictionary representation of a load balancer
         :param vip_dict: Dictionary representation of a vip
-        :returns: octava.common.data_models.LoadBalancer
+        :returns: octavia.common.data_models.LoadBalancer
         """
         with session.begin(subtransactions=True):
             if not lb_dict.get('id'):
@@ -741,6 +747,18 @@ class MemberRepository(BaseRepository):
         """Batch deletes members from a pool."""
         self.delete_batch(session, member_ids)
 
+    def update_pool_members(self, session, pool_id, **model_kwargs):
+        """Updates all of the members of a pool.
+
+        :param session: A Sql Alchemy database session.
+        :param pool_id: ID of the pool to update members on.
+        :param model_kwargs: Entity attributes that should be updates.
+        :returns: octavia.common.data_model
+        """
+        with session.begin(subtransactions=True):
+            session.query(self.model_class).filter_by(
+                pool_id=pool_id).update(model_kwargs)
+
 
 class ListenerRepository(BaseRepository):
     model_class = models.Listener
@@ -1290,6 +1308,12 @@ class L7PolicyRepository(BaseRepository):
 class QuotasRepository(BaseRepository):
     model_class = models.Quotas
 
+    # This is used with an autocommit session (non-lock_session)
+    # Since this is for the initial quota record creation it locks the table
+    # which can lead to recoverable deadlocks. Thus we use the deadlock
+    # retry wrapper here. This may not be appropriate for other sessions
+    # and or queries. Use with caution.
+    @oslo_db_api.wrap_db_retry(max_retries=5, retry_on_deadlock=True)
     def update(self, session, project_id, **model_kwargs):
         with session.begin(subtransactions=True):
             kwargs_quota = model_kwargs['quota']
