@@ -108,11 +108,12 @@ class NoopManager(object):
             compute_id, 'get_plugged_networks')
         return []
 
-    def update_vip(self, loadbalancer):
-        LOG.debug("Network %s no-op, update_vip loadbalancer %s",
-                  self.__class__.__name__, loadbalancer)
+    def update_vip(self, loadbalancer, for_delete=False):
+        LOG.debug("Network %s no-op, update_vip loadbalancer %s "
+                  "with for delete %s",
+                  self.__class__.__name__, loadbalancer, for_delete)
         self.networkconfigconfig[loadbalancer.id] = (
-            loadbalancer, 'update_vip')
+            loadbalancer, for_delete, 'update_vip')
 
     def get_network(self, network_id):
         LOG.debug("Network %s no-op, get_network network_id %s",
@@ -186,10 +187,39 @@ class NoopManager(object):
         self.networkconfigconfig[(loadbalancer.id)] = (
             loadbalancer, 'get_network_configs')
 
+        amp_configs = {}
+        for amp in loadbalancer.amphorae:
+            vrrp_port = self.get_port(amp.vrrp_port_id)
+            ha_port = self.get_port(amp.ha_port_id)
+            amp_configs[amp.id] = network_models.AmphoraNetworkConfig(
+                amphora=amp,
+                vip_subnet=self.get_subnet(loadbalancer.vip.subnet_id),
+                vip_port=self.get_port(loadbalancer.vip.port_id),
+                vrrp_subnet=self.get_subnet(
+                    vrrp_port.get_subnet_id(amp.vrrp_ip)),
+                vrrp_port=vrrp_port,
+                ha_subnet=self.get_subnet(
+                    ha_port.get_subnet_id(amp.ha_ip)),
+                ha_port=ha_port)
+        return amp_configs
+
     def wait_for_port_detach(self, amphora):
         LOG.debug("failover %s no-op, wait_for_port_detach, amphora id %s",
                   self.__class__.__name__, amphora.id)
 
+    def get_qos_policy(self, qos_policy_id):
+        LOG.debug("Qos Policy %s no-op, get_qos_policy qos_policy_id %s",
+                  self.__class__.__name__, qos_policy_id)
+        self.networkconfigconfig[qos_policy_id] = (qos_policy_id,
+                                                   'get_qos_policy')
+        return qos_policy_id
+
+    def apply_qos_on_port(self, qos_id, port_id):
+        LOG.debug("Network %s no-op, apply_qos_on_port qos_id %s, port_id "
+                  "%s", self.__class__.__name__, qos_id, port_id)
+        self.networkconfigconfig[(qos_id, port_id)] = (
+            qos_id, port_id, 'apply_qos_on_port')
+
 
 class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
     def __init__(self):
@@ -218,8 +248,8 @@ class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
     def get_plugged_networks(self, amphora_id):
         return self.driver.get_plugged_networks(amphora_id)
 
-    def update_vip(self, loadbalancer):
-        self.driver.update_vip(loadbalancer)
+    def update_vip(self, loadbalancer, for_delete=False):
+        self.driver.update_vip(loadbalancer, for_delete)
 
     def get_network(self, network_id):
         return self.driver.get_network(network_id)
@@ -230,6 +260,9 @@ class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
     def get_port(self, port_id):
         return self.driver.get_port(port_id)
 
+    def get_qos_policy(self, qos_policy_id):
+        return self.driver.get_qos_policy(qos_policy_id)
+
     def get_network_by_name(self, network_name):
         return self.driver.get_network_by_name(network_name)
 
@@ -253,3 +286,6 @@ class NoopNetworkDriver(driver_base.AbstractNetworkDriver):
 
     def wait_for_port_detach(self, amphora):
         self.driver.wait_for_port_detach(amphora)
+
+    def apply_qos_on_port(self, qos_id, port_id):
+        self.driver.apply_qos_on_port(qos_id, port_id)
